import expect from 'expect.js';
import { KibanaMap } from '../kibana_map';
import { KibanaMapLayer } from '../kibana_map_layer';
import L from 'leaflet';

describe('kibana_map tests', function () {

  let domNode;
  let kibanaMap;

  function setupDOM() {
    domNode = document.createElement('div');
    domNode.style.top = '0';
    domNode.style.left = '0';
    domNode.style.width = '512px';
    domNode.style.height = '512px';
    domNode.style.position = 'fixed';
    domNode.style['pointer-events'] = 'none';
    document.body.appendChild(domNode);
  }

  function teardownDOM() {
    domNode.innerHTML = '';
    document.body.removeChild(domNode);
  }


  describe('KibanaMap - basics', function () {

    beforeEach(async function () {
      setupDOM();
      kibanaMap = new KibanaMap(domNode, {
        minZoom: 1,
        maxZoom: 10,
        center: [0,0],
        zoom: 0
      });
    });

    afterEach(function () {
      kibanaMap.destroy();
      teardownDOM();
    });

    it('should instantiate at zoom level 2', function () {
      const bounds = kibanaMap.getBounds();
      expect(bounds.bottom_right.lon).to.equal(90);
      expect(bounds.top_left.lon).to.equal(-90);
      expect(kibanaMap.getCenter().lon).to.equal(0);
      expect(kibanaMap.getCenter().lat).to.equal(0);
      expect(kibanaMap.getZoomLevel()).to.equal(2);
    });

    it('should resize to fit container', function () {

      kibanaMap.setZoomLevel(2);
      expect(kibanaMap.getCenter().lon).to.equal(0);
      expect(kibanaMap.getCenter().lat).to.equal(0);

      domNode.style.width = '1024px';
      domNode.style.height = '1024px';
      kibanaMap.resize();

      expect(kibanaMap.getCenter().lon).to.equal(0);
      expect(kibanaMap.getCenter().lat).to.equal(0);
      const bounds = kibanaMap.getBounds();
      expect(bounds.bottom_right.lon).to.equal(180);
      expect(bounds.top_left.lon).to.equal(-180);

    });
  });


  describe('KibanaMap - attributions', function () {


    beforeEach(async function () {
      setupDOM();
      kibanaMap = new KibanaMap(domNode, {
        minZoom: 1,
        maxZoom: 10,
        center: [0, 0],
        zoom: 0
      });
    });

    afterEach(function () {
      kibanaMap.destroy();
      teardownDOM();
    });

    function makeMockLayer(attribution) {
      const layer = new KibanaMapLayer();
      layer._attribution = attribution;
      layer._leafletLayer = L.geoJson(null);
      return layer;
    }

    it('should update attributions correctly', function () {
      kibanaMap.addLayer(makeMockLayer('foo|bar'));
      expect(domNode.querySelectorAll('.leaflet-control-attribution')[0].innerHTML).to.equal('foo, bar');

      kibanaMap.addLayer(makeMockLayer('bar'));
      expect(domNode.querySelectorAll('.leaflet-control-attribution')[0].innerHTML).to.equal('foo, bar');

      const layer = makeMockLayer('bar,stool');
      kibanaMap.addLayer(layer);
      expect(domNode.querySelectorAll('.leaflet-control-attribution')[0].innerHTML).to.equal('foo, bar, stool');

      kibanaMap.removeLayer(layer);
      expect(domNode.querySelectorAll('.leaflet-control-attribution')[0].innerHTML).to.equal('foo, bar');


    });

  });

  describe('KibanaMap - baseLayer', function () {

    beforeEach(async function () {
      setupDOM();
      kibanaMap = new KibanaMap(domNode, {
        minZoom: 1,
        maxZoom: 10,
        center: [0,0],
        zoom: 0
      });
    });

    afterEach(function () {
      kibanaMap.destroy();
      teardownDOM();
    });


    it('TMS', async function () {

      const options = {
        'url': 'https://tiles-stage.elastic.co/v2/default/{z}/{x}/{y}.png?elastic_tile_service_tos=agree&my_app_name=kibana',
        'minZoom': 0,
        'maxZoom': 12,
        'attribution': '© [Elastic Maps Service](https://www.elastic.co/elastic-maps-service)'
      };


      return new Promise(function (resolve) {
        kibanaMap.on('baseLayer:loaded', () => {
          resolve();
        });
        kibanaMap.setBaseLayer({
          baseLayerType: 'tms',
          options: options
        });
      });
    });

    it('WMS', async function () {

      const options = {
        url: 'https://basemap.nationalmap.gov/arcgis/services/USGSTopo/ MapServer/WMSServer',
        version: '1.3.0',
        layers: '0',
        format: 'image/png',
        transparent: true,
        attribution: 'Maps provided by USGS',
        styles: '',
        minZoom: 1,
        maxZoom: 18
      };


      return new Promise(function (resolve) {
        kibanaMap.on('baseLayer:loaded', () => {
          resolve();
        });
        kibanaMap.setBaseLayer({
          baseLayerType: 'wms',
          options: options
        });
      });
    });

  });


});
